;******************************************************************************
;   This file is a basic template for creating relocatable assembly code for  *
;   a PIC18F4320. Copy this file into your project directory and modify or    *
;   add to it as needed. Create a project with MPLINK as the language tool    *
;   for the hex file. Add this file and the 18F4320.LKR file to the project.  *
;                                                                             *
;   The PIC18FXXXX architecture allows two interrupt configurations. This     *
;   template code is written for priority interrupt levels and the IPEN bit   *
;   in the RCON register must be set to enable priority levels. If IPEN is    *
;   left in its default zero state, only the interrupt vector at 0x008 will   *
;   be used and the WREG_TEMP, BSR_TEMP and STATUS_TEMP variables will not    *
;   be needed.                                                                *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on the         *
;   features of the assembler and linker.                                     *
;                                                                             *
;   Refer to the PIC18F2220/2320/4220/4320 Data Sheet for additional          *
;   information on the architecture and instruction set.                      *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:                                                                *
;    Date:                                                                    *
;    File Version:                                                            *
;                                                                             *
;    Author:                                                                  *
;    Company:                                                                 *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files required:         P18F4320.INC                                     *
;                            18F4320.LKR                                      *
;                                                                             *
;******************************************************************************

	LIST P=18F4320, F=INHX32 ;directive to define processor and file format
	#include <P18F4320.INC>	 ;processor specific variable definitions

;******************************************************************************
;Configuration bits
; The __CONFIG directive defines configuration data within the .ASM file.
; The labels following the directive are defined in the P18F4320.INC file.
; The PIC18F2220/2320/4220/4320 Data Sheet explains the functions of the
; configuration bits. Change the following lines to suit your application.

	__CONFIG  _CONFIG1H, _IESO_OFF_1H & _FSCM_OFF_1H & _HS_OSC_1H
	__CONFIG  _CONFIG2L, _PWRT_OFF_2L & _BOR_OFF_2L
	__CONFIG  _CONFIG2H, _WDT_OFF_2H
	__CONFIG  _CONFIG3H, _MCLRE_OFF_3H & _PBAD_ANA_3H & _CCP2MX_C1_3H
	__CONFIG  _CONFIG4L, _DEBUG_OFF_4L & _LVP_OFF_4L & _STVR_OFF_4L
	__CONFIG  _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L
	__CONFIG  _CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H
	__CONFIG  _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L
	__CONFIG  _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H
	__CONFIG  _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L
	__CONFIG  _CONFIG7H, _EBTRB_OFF_7H

;******************************************************************************
;Variable definitions
; These variables are only needed if low priority interrupts are used. 
; More variables may be needed to store other special function registers used
; in the interrupt routines.

; portB controls car: <fwd><fwd#><rev><rev#><lft><lft#><rgt><rgt#>
FWD		EQU		7
FWD#	EQU		6
REV		EQU		5
REV#	EQU		4
LFT		EQU		3
LFT#	EQU		2
RGT		EQU		1
RGT#	EQU		0

		UDATA

WREG_TEMP	RES	1	;variable in RAM for context saving 
TEMP		RES 1
STATUS_TEMP	RES	1	;variable in RAM for context saving
BSR_TEMP	RES	1	;variable in RAM for context saving

		UDATA_ACS

EXAMPLE		RES	1	;example of a variable in access RAM

;******************************************************************************
;EEPROM data
; Data to be programmed into the Data EEPROM is defined here

DATA_EEPROM	CODE	0xF00000

		DE	"Test Data",0,1,2,3,4,5

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

RESET_VECTOR	CODE	0x0000

		goto	Main		;go to start of main code

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

HI_INT_VECTOR	CODE	0x0008

		bra	HighInt		;go to high priority interrupt routine

;******************************************************************************
;Low priority interrupt vector
; This code will start executing when a low priority interrupt occurs.
; This code can be removed if low priority interrupts are not used.

LOW_INT_VECTOR	CODE	0x0018

		bra	LowInt		;go to low priority interrupt routine

;******************************************************************************
;High priority interrupt routine
; The high priority interrupt code is placed here.

		CODE

HighInt:

;	*** high priority interrupt code goes here ***


		retfie	FAST

;******************************************************************************
;Low priority interrupt routine
; The low priority interrupt code is placed here.
; This code can be removed if low priority interrupts are not used.

LowInt:
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register

;	*** low priority interrupt code goes here ***


		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;******************************************************************************
;Start of main program
; The main program code is placed here.

Main:
	; configure I/O pins
	CLRF	TRISC		; all outputs
	BSF		TRISC, RC7	; input
	CLRF	TRISB
	CLRF	TRISD

	; set low 5 bits of B to digital IO
	MOVLW	0xF
	IORWF	ADCON1
	
	BCF		PORTD, RD1

	BCF		PORTD, RD0
	NOP
	NOP
	NOP
	NOP
	BSF		PORTD, RD0
	NOP
	NOP
	NOP
	NOP
	BCF		PORTD, RD0
	
	; initialize BRG for UART
	BSF		TXSTA, BRGH	; use high speed UART
	MOVLW	0x81		; osc=20MHz => 96200Kbps
	MOVWF	SPBRG
	
	; initialize UART
	BCF		TXSTA, SYNC		; async
	BSF		TXSTA, TXEN		; enable transmit
	BSF		RCSTA, SPEN		; set RC7/RC6 as Rx/Tx
	BSF		RCSTA, CREN
	
	;<fwd><fwd#><rev><rev#><lft><lft#><rgt><rgt#>
	MOVLW	b'00000000'
	MOVWF	PORTB

	; monitor Rx for recieve
mainloop:
	BTFSS	PIR1, RCIF
	GOTO	mainloop
	
	; recieve command
	MOVF	RCREG, W
	MOVWF	WREG_TEMP
	
	; process command
	MOVF	WREG_TEMP, W
	SUBLW	'F'
	BTFSC	STATUS, Z
	GOTO	goforward

	MOVF	WREG_TEMP, W
	SUBLW	'B'
	BTFSC	STATUS, Z
	GOTO	gobackward
	
	MOVF	WREG_TEMP, W
	SUBLW	'S'
	BTFSC	STATUS, Z
	GOTO	gostop
	
	MOVF	WREG_TEMP, W
	SUBLW	'L'
	BTFSC	STATUS, Z
	GOTO	goleft
	
	MOVF	WREG_TEMP, W
	SUBLW	'R'
	BTFSC	STATUS, Z
	GOTO	goright
	
	MOVF	WREG_TEMP, W
	SUBLW	'C'
	BTFSC	STATUS, Z
	GOTO	gocenter
	
	MOVF	WREG_TEMP, W
	SUBLW	'H'
	BTFSC	STATUS, Z
	GOTO	gobuzz
	
	MOVF	WREG_TEMP, W
	SUBLW	'Z'
	BTFSC	STATUS, Z
	GOTO	gostopbuzz

	GOTO	mainloop

gobuzz:
	BSF		PORTD, RD1
	GOTO	mainloop
	
gostopbuzz:
	BCF		PORTD, RD1
	GOTO	mainloop
	
	; portB controls car: <fwd><fwd#><rev><rev#><lft><lft#><rgt><rgt#>
goforward:
	; if motor is going backward, stop the motor
	BCF		PORTB, FWD
	BCF		PORTB, FWD#
	BCF		PORTB, REV
	BCF		PORTB, REV#
	
	; TODO: wait?
	NOP
	NOP
	
	; start motor going forward
	BSF		PORTB, FWD
	BCF		PORTB, FWD#
	BCF		PORTB, REV
	BSF		PORTB, REV#
	
	GOTO	mainloop

gobackward:
	; if motor is going foward, stop the motor
	BCF		PORTB, REV
	BCF		PORTB, REV#
	BCF		PORTB, FWD
	BCF		PORTB, FWD#
	
	; TODO: wait?
	NOP
	NOP
	
	; start the motor going backward
	BSF		PORTB, REV
	BCF		PORTB, REV#
	BCF		PORTB, FWD
	BSF		PORTB, FWD#
	
	GOTO	mainloop

gostop:
	BCF		PORTB, FWD
	BCF		PORTB, REV
	BCF		PORTB, FWD#
	BCF		PORTB, REV#
	
	GOTO	mainloop

goleft:
	; if motor is going right, center the motor
	BCF		PORTB, LFT
	BCF		PORTB, LFT#
	BCF		PORTB, RGT
	BCF		PORTB, RGT#
	
	; TODO: wait?
	NOP
	NOP
	
	; start the motor going left
	BSF		PORTB, LFT
	BCF		PORTB, LFT#
	BCF		PORTB, RGT
	BSF		PORTB, RGT#
	
	GOTO	mainloop

goright:
	; if motor is going left, center the motor
	BCF		PORTB, RGT
	BCF		PORTB, RGT#
	BCF		PORTB, LFT
	BCF		PORTB, LFT#
	
	; TODO: wait?
	NOP
	NOP
	
	; start the motor going right
	BSF		PORTB, RGT
	BCF		PORTB, RGT#
	BCF		PORTB, LFT
	BSF		PORTB, LFT#
	
	GOTO mainloop

gocenter:
	BCF		PORTB, RGT
	BCF		PORTB, LFT
	BCF		PORTB, RGT#
	BCF		PORTB, LFT#
	
	GOTO	mainloop
	
	END

	ORG	0x700
; BlueRadios AT Commands used by the PIC
BR_AT_GET_INIT	db	0x03, "+++"
BR_AT_GET_AT	db	0x02, "AT"
BR_AT_GET_VER	db	0x0A, "ATVER,ver1"
BR_AT_GET_MAC	db	0x06, "ATSI,2"
BR_AT_GET_CONN	db	0x06, "ATSI,3"
BR_AT_GET_DBS	db	0x07, "ATSI,14"

BR_AT_SET_NAME	db	0x0A, "ATSN,BTCAR"
BR_AT_SET_BAUD	db	0x11, "ATSW20,3775,0,0,0"

TEST_STRING		db	0x0E, "Hello World!", 0x13, 0x10

RX_BUFFER		db	0x00, "                "
